/*
 * Created by Jesse Phillips
 * Created on March 5, 2006
 */

/**
 * An attempt to create a fuzzy logic AI
 * based on the tags files have.
 *
 * This test will use fake files read in
 * from a file.
 */

import std.string;
import std.file;
import std.stdio;

int main() {
	char[] input2 = cast(char[])read("files.in");
	char[][] input = splitlines(input2);
	TagGroup[] list = new TagGroup[50];
	TagGroup[Tag] category;
	Tag[] tags = new Tag[100];

	int i = 0;
	int j = 0;
	foreach(char[] c; input) {
		char[][] single = split(c, "; ");
		/// Build a list of all the tags, and count repeats.
		foreach(char[] t; single) {
			bool has = false;
			foreach(v;tags) {
				if(v is null) continue;
				if(v.name == t) {
					has = true;
					v.count++;
				}
			}
			if(!has)
				if(t != "" && t != " ") {
					tags[i++] = new Tag(t);
					category[tags[i-1]] = new TagGroup(0);
				}
		}

		/// Build an array of all the grouped tags.
		/// Each files tag set is a group.
		TagGroup group = new TagGroup();
		foreach(char[] ct; single)
			foreach(t; tags)
				if(t !is null)
				if(t.name == ct && ct != "" && ct != "\n") {
					group.addTag(t);
					break;
				}
		list[j++] = group;
	}
	
	// Print the list of tags and their count.
	debug(9) {
		foreach(t;tags)
			if(t !is null)
			writefln("'%s' has %s", t.name, t.count);
	}

	debug(9) {
		foreach(tg; list)
			if(tg !is null)
				foreach(t; tg.tags)
					if(t !is null)
				writefln(t);
	}

	/**
	 * Run through all the tags to find out what tags
	 * occur most often with that tag. And group them.
	 *
	 * This loop is probably the least efficient way to
	 * accomplish the task. But it works and needs expanded.
	 * The task is to find out what tags should appear as
	 * sub tags to that tag. The current "AI" will tag
	 * the tags that occur half as often, as the selected tag,
	 * or more.
	 */
	int index = 0;
	foreach(cat;tags) {
		if(cat !is null) {
			debug(6) writefln("\n%s: %s", cat, cat.count);
			foreach(find;tags)
				if(find !is null) {
					int numOfTags = countTagWith(cat, find, list);
					debug(8) {
						writefln("Found %s in %s", numOfTags, find);
					}else
						if(find.count <= (cat.count/2+1)) {
							if(numOfTags > 0) {
								debug(7) writefln("\tNum of tgs in cat %s:\tNum of cat %s",
													numOfTags, (cat.count/2));
								debug(6) writefln(find);
								debug(5) writefln("Adding: %s", find);
								category[cat].addTag(find);
							   if(numOfTags <= cat.count) {
                                 				category[find].level = category[find].level + 1;
                                
                              				    }
      							}
						}
				}
		}
	}


	debug(3) {
		Tag[] keys = category.keys;

        
        foreach(key; keys)
                     debug(4)
                             writefln("%s| %s	\t: %s\n", key, category[key].level, category[key]);
                     else
                         if(category[key].level == 0) {
                                writefln("/%s", key);
                                foreach(Tag sub; category[key].getTags())
                                     if(sub !is null)
                                          if(category[sub].level == 1)
                                             writefln("/%s/%s", key, sub);
                         }

	}

			
	
	return 0;
}
int countTagWith(Tag cat, Tag find, TagGroup[] list) {
    int count = 0;
	if(cat.name != find.name)
		foreach(tg; list) {
			if(tg is null) continue;
			Tag result = tg.hasTag(cat);
			if(result !is null)
				if(tg.hasTag(find) !is null)
					count++;
		}
	return count;
}

class Tag {
public:
	char[] name; int count = 1; 

	this(char[] name) {
		this.name = name;
	}

	char[] toString() {
		return name;
	}
}

public class TagGroup {
private:
	Tag[] tags;
	int tagNum = 0;

public:
       int level = -1;
       this() {
		tags = new Tag[50];
	}
	this(int lvl) {
	         tags = new Tag[50];
	         level = lvl;
           }
    Tag[] getTags() {
              return tags;
    }

	void addTag(Tag tag) {
		tags[tagNum++] = tag;
	}

//	void addTags(char[][] tags) {
//		foreach(char[] s; tags)
//			this.tags[tagNum++] = s;
//	}

	Tag hasTag(Tag tag) {
		foreach(t; tags)
			if(t !is null && tag !is null)
			if(t.name == tag.name)
				return t;
		return null;
	}

	Tag hasTag(char[] tag) {
		foreach(t; tags)
			if(t !is null)
			if(t.name == tag)
				return t;
		return null;
	}

	char[] toString() {
		char[] result = "";
		if(tags[0] !is null)
				result ~= tags[0].name;
		foreach(t;tags)
			if(t !is null && t.name != tags[0].name)
				result ~= ", "~t.name;
		return result;
	}
}
